package util;

public class List<ElementType> {
	private class ListElement {
		public ElementType content;
		public ListElement next;
		public ListElement previous;
		public ListElement(ElementType element) {
			content = element;
			next = previous = null;
		}
	}
	private ListElement first, last;
	private int numberOfElements;
	public List() {
		first = last = null;
		numberOfElements = 0;
	}
	public void add(ElementType element) {
		ListElement newListElement = new ListElement(element);
		if (first == null) {
			first = last = newListElement;
		} else {
			last.next = newListElement;
			newListElement.previous = last;
		}
		last = newListElement;
		numberOfElements++;
	}
	private void remove(ListElement current) {
		if (current == first && current == last) {
			first = last = null;
			numberOfElements = 0;
		} else {
			if (current != first && current != last) {
				current.previous.next = current.next;
				current.next.previous = current.previous;
			}
			if (current == first) {
				first = current.next;
				first.previous = null;
			}
			if (current == last) {
				last = current.previous;
				last.next = null;
			}
			numberOfElements--;
		}
		current.next = current.previous = null;
	}
	public void remove(ElementType element) {
		ListElement current = first;
		while (current != null && current.content != element) {
			current = current.next;
		}
		if (current != null) remove(current);
	}
	public void remove(int index) {
		ListElement current;
		if (index < size() / 2) {
			current = first;
			while (current != null && index > 0) {
				current = current.next;
				index--;
			}
		} else {
			index = size() - index - 1;
			current = last;
			while (current != null && index > 0) {
				current = current.previous;
				index--;
			}
		}
		remove(current);
	}
	public int size() {
		return numberOfElements;
	}
	private ElementType getFromBeginning(int index) {
		ListElement current = first;
		while (current != null && index > 0) {
			System.err.print(" " + current.content + " ");
			current = current.next;
			index--;
		}
		System.err.println(" " + current.content);
		return current.content;
	}
	private ElementType getFromEnd(int index) {
		ListElement current = last;
		while (current != null && index > 0) {
			System.err.print(" " + current.content + " ");
			current = current.previous;
			index--;
		}
		return current.content;
	}
	public ElementType get(int index) {
		if (index < size() / 2)
			return getFromBeginning(index);
		return getFromEnd(size() - index - 1);
	}
	public boolean isEmpty() {
		return first == null;
	}
	public boolean contents(ElementType element) {
		ListElement current = first;
		while (current != null && current.content != element) {
			current = current.next;
		}
		return (current != null);
	}
}
